#include <Adafruit_GFX.h>
#include <Adafruit_NeoPixel.h>

#include <EEPROM.h>
#include "printf.h"
#include <Streaming.h>

/*
* Getting Started example sketch for nRF24L01+ radios
* This is a very basic example of how to send data from one node to another
* Updated: Dec 2014 by TMRh20
*/

#include <SPI.h>
#include "RF24.h"

/****************** User Config ***************************/
/***      Set this radio as radio number 0 or 1         ***/
uint8_t radioNumber;

/* Hardware configuration: Set up nRF24L01 radio on SPI bus plus pins 7 & 8 */
RF24 radio(9,10);
/**********************************************************/

byte addresses[][6] = {"1Node","2Node"};

// Used to control whether this node is sending or receiving
uint8_t role;

#define PAYLOAD_SIZE 4

Adafruit_NeoPixel pixels = Adafruit_NeoPixel(1, 4, NEO_GRB + NEO_KHZ800);

void setup() {
  printf_begin();
  
  role = EEPROM.read(0);
  radioNumber = EEPROM.read(0);
  //EEPROM.write(0,0); // 0 = PongBack
  
  Serial.begin(57600);
  printf_begin();
  //Serial.println(F("RF24/examples/GettingStarted"));
  //Serial.println(F("*** PRESS 'T' to begin transmitting to the other node"));
  //EEPROM.write(0,0); // 0 = PongBack

  Serial << "radioNumber: " << radioNumber << ", role: " << role << endl;
  
  radio.begin(); 
  radio.setDataRate(RF24_250KBPS);

  //settings[autoack] = 0;
  //settings[retry] = 15;
  //settings[delay_rf] = 15;
  //settings[delay_loop] = 100; 
  
  radio.setAutoAck(0);
  radio.setRetries(15,15);
  
  radio.setPALevel(RF24_PA_HIGH);
  radio.setPayloadSize(PAYLOAD_SIZE);
  
  // Open a writing and reading pipe on each radio, with opposite addresses
  if(radioNumber){
    radio.openWritingPipe(addresses[1]);
    radio.openReadingPipe(1,addresses[0]);
  }else{
    radio.openWritingPipe(addresses[0]);
    radio.openReadingPipe(1,addresses[1]);
  }
  
  // Start the radio listening for data
  radio.startListening();
  radio.setChannel(76);

  if(role == 1)
  {

    pixels.begin();
    pixels.setBrightness(120);
    
    pixels.setPixelColor(0,pixels.Color(255,0,0));
    pixels.show();
    delay(500);
    pixels.setPixelColor(0,pixels.Color(0,255,0));
    pixels.show();
    delay(500);
    pixels.setPixelColor(0,pixels.Color(0,0,255));
    pixels.show();
    delay(500);
  }
}

void loop() {
 
/****************** Ping Out Role ***************************/  
if (role == 1)
{
    char buff[PAYLOAD_SIZE];
    sprintf(buff,"%3d",millis()%1000);
    
    rf_write(buff,PAYLOAD_SIZE);
    delay(1000);
}



/****************** Pong Back Role ***************************/

  if ( role == 0 )
  {
      static uint8_t rx_buffer[32];
      static uint8_t rx_buffer_len = 0;

      byte pipeNo = 0;

      bool timeout = false;
      unsigned long started_waiting_at = millis();

      while ( ! radio.available(&pipeNo) ) {                            // While nothing is received
        if (millis() - started_waiting_at > 1200 ) {           // If waited longer than 200ms, indicate timeout and exit while loop
          Serial.write('t');
          timeout = true;
          break;
        }
      }
      
      if(timeout == false)
      {            
        radio.read(rx_buffer, PAYLOAD_SIZE);
        radio.stopListening();
        
        radio.write(rx_buffer, PAYLOAD_SIZE);  
        radio.startListening();
      }
 }

} // Loop

