#include "printf.h"
#include "RF24.h"
#include "nRF24L01.h"

#include <Adafruit_GFX.h>
#include <Adafruit_NeoPixel.h>

// Gamma correction improves appearance of midrange colors
const uint8_t PROGMEM gamma8[] = {
      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  1,  1,  1,  1,  1,
      1,  1,  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  3,  3,  3,  3,
      3,  3,  4,  4,  4,  4,  5,  5,  5,  5,  5,  6,  6,  6,  6,  7,
      7,  7,  8,  8,  8,  9,  9,  9, 10, 10, 10, 11, 11, 11, 12, 12,
     13, 13, 13, 14, 14, 15, 15, 16, 16, 17, 17, 18, 18, 19, 19, 20,
     20, 21, 21, 22, 22, 23, 24, 24, 25, 25, 26, 27, 27, 28, 29, 29,
     30, 31, 31, 32, 33, 34, 34, 35, 36, 37, 38, 38, 39, 40, 41, 42,
     42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57,
     58, 59, 60, 61, 62, 63, 64, 65, 66, 68, 69, 70, 71, 72, 73, 75,
     76, 77, 78, 80, 81, 82, 84, 85, 86, 88, 89, 90, 92, 93, 94, 96,
     97, 99,100,102,103,105,106,108,109,111,112,114,115,117,119,120,
    122,124,125,127,129,130,132,134,136,137,139,141,143,145,146,148,
    150,152,154,156,158,160,162,164,166,168,170,172,174,176,178,180,
    182,184,186,188,191,193,195,197,199,202,204,206,209,211,213,215,
    218,220,223,225,227,230,232,235,237,240,242,245,247,250,252,255
};

bool rf_write(void * src, uint8_t len)
{
  bool timeout = false;
  uint8_t rpd;

  // Tracking and Buffer
  static uint8_t tx_buffer[32];
  static uint8_t rx_buffer[32];
  static uint8_t rx_buffer_len = 0;

  unsigned long start_time = micros();

  radio.stopListening();
  bool write_success_autoack = radio.write( src, len );
  uint8_t ss = radio.read_register(OBSERVE_TX);

  radio.startListening();  
  Serial << "| loss: " << _HEX(0x0F & ss) << " | " << readVcc() << endl;  
  
  unsigned long started_waiting_at = millis();
  while ( ! radio.available() ) {                            
    if (millis() - started_waiting_at > 100 ) {           
      timeout = true;
      break;
    }
  }
  ss = radio.read_register(OBSERVE_TX);
  rpd = radio.testRPD();
  Serial << " | rpd: " << rpd;

  // COLOR LED
  uint32_t color_start = pixels.Color(0,255,0); // Green
  uint32_t color_end   = pixels.Color(255,0,0);

  //new_color = color_end;
  uint32_t newColor = fade((ss & 0x0F),0x0F,color_start,color_end);

  // Purple == good
  if(rpd == 1)
    newColor = pixels.Color(143,0,255);

  // Timeout = red
  if(timeout)
    newColor = pixels.Color(255,0,0);

  // good reception, still loss?!
  if(rpd == 1 && ((0x0F & ss) > 0))
    newColor = pixels.Color(50,205,50);
    
  //Serial << " newColor: " << _HEX(newColor) << endl;
  pixels.setPixelColor(0,newColor);
  pixels.show();
  
  if (!timeout)
  {
    radio.read(rx_buffer, PAYLOAD_SIZE);
    unsigned long end_time = micros();
    Serial << " | rx: " << (char*)rx_buffer;
    return true;
  }
  else
  {
    Serial << " | rx: --- ";
    return false;
  }
}

uint32_t fade(uint8_t steptarget, uint8_t steps, uint32_t startColor, uint32_t endColor) {

    uint8_t  startColor_getR = (startColor >> 16) & 0xFF;
    uint8_t  startColor_getG = (startColor >>  8) & 0xFF;
    uint8_t  startColor_getB = startColor & 0xFF;

    uint8_t  endColor_getR = (endColor >> 16) & 0xFF;
    uint8_t  endColor_getG = (endColor >>  8) & 0xFF;
    uint8_t  endColor_getB = endColor & 0xFF;
  
    int16_t redDiff = endColor_getR - startColor_getR;
    int16_t greenDiff = endColor_getG - startColor_getG;
    int16_t blueDiff = endColor_getB - startColor_getB;

    int16_t redValue, greenValue, blueValue;

    redValue = (int16_t)startColor_getR + (redDiff * steptarget / steps);
    greenValue = (int16_t)startColor_getG + (greenDiff * steptarget / steps);
    blueValue = (int16_t)startColor_getB + (blueDiff * steptarget / steps);
    /*  
    for (int16_t i = 0 ; i < steptarget - 1 ; ++i) {
        redValue = (int16_t)startColor_getR + (redDiff * i / steps);
        greenValue = (int16_t)startColor_getG + (greenDiff * i / steps);
        blueValue = (int16_t)startColor_getB + (blueDiff * i / steps);
       
        Serial.print(redValue);
        Serial.print("\t");
        Serial.print(greenValue);
        Serial.print("\t");
        Serial.print(blueValue);
        Serial.println("\t");
        
    }
    */
    return pixels.Color(redValue,greenValue,blueValue);
}

long readVcc() {
  // Read 1.1V reference against AVcc
  // set the reference to Vcc and the measurement to the internal 1.1V reference
  #if defined(__AVR_ATmega32U4__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
    ADMUX = _BV(REFS0) | _BV(MUX4) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  #elif defined (__AVR_ATtiny24__) || defined(__AVR_ATtiny44__) || defined(__AVR_ATtiny84__)
    ADMUX = _BV(MUX5) | _BV(MUX0);
  #elif defined (__AVR_ATtiny25__) || defined(__AVR_ATtiny45__) || defined(__AVR_ATtiny85__)
    ADMUX = _BV(MUX3) | _BV(MUX2);
  #else
    ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  #endif  

  delay(2); // Wait for Vref to settle
  ADCSRA |= _BV(ADSC); // Start conversion
  while (bit_is_set(ADCSRA,ADSC)); // measuring

  uint8_t low  = ADCL; // must read ADCL first - it then locks ADCH  
  uint8_t high = ADCH; // unlocks both

  long result = (high<<8) | low;

  result = 1125300L / result; // Calculate Vcc (in mV); 1125300 = 1.1*1023*1000
  //result *= 1.0637;
  return result; // Vcc in millivolts
}

#include <stdarg.h>
void p(const __FlashStringHelper *fmt, ... ){
  char buf[128]; // resulting string limited to 128 chars
  va_list args;
  va_start (args, fmt);
#ifdef __AVR__
  vsnprintf_P(buf, sizeof(buf), (const char *)fmt, args); // progmem for AVR
#else
  vsnprintf(buf, sizeof(buf), (const char *)fmt, args); // for the rest of the world
#endif
  va_end(args);
  Serial.print(buf);
}
